import java.awt.Shape;
import java.awt.geom.Rectangle2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * Eine KonferenzTischGruppe
 * 
 * @author Claus Albowski
 * @version 2  (12.05.09)
 */
public class Gruppe extends Moebel
{
    protected ArrayList<Moebel> moebelListe;
    protected Rectangle2D umriss;

    /**
    * Erzeuge eine neue Gruppe.
    */
    public Gruppe(int x, int y, String f, int o) {
        xPosition = x;
        yPosition = y;
        farbe = f;
        orientierung = o;
        istSichtbar = false;
        init();
    }
    
    /**
    * erzeugt die Gruppe und ihre Elemente.
    */
    protected void init() {
        moebelListe = new ArrayList<Moebel>();
        breite = 0;
        tiefe  = 0;
        umriss= null;
    }
    /**
     * berschreibt die Methode von Moebel
     */
    protected void setzeAuswahl(boolean wert){
      for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
      it.next().setzeAuswahl(wert);
    }
    
    /**
     * setzt die Gruppentransformation der Kindobjekte.
     */
    protected void setzeGruppenTransformation(){
        if (moebelListe!=null){
            AffineTransform t = new AffineTransform();
            if (gruppenTransformation!=null) t.concatenate(gruppenTransformation);
            //System.out.println("x: "+xPosition+"  y: "+yPosition);
            t.translate(xPosition, yPosition);
            // relative Orientierung nach Definition:
            t.rotate(Math.toRadians(orientierung),
                   umriss.getX()+umriss.getWidth()/2,
                   umriss.getY()+umriss.getHeight()/2);
            // vorgegebene Rotation durch eine Gruppe:
            for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
            it.next().setzeGruppenTransformation(t);
        }
    }
    
    /**
     * aktualisiert den Umriss.
     */
    private void aktualisiereUmriss(){
        Shape figur=gibAktuelleFigur();
        umriss=figur.getBounds2D();
        breite=(int)umriss.getWidth();
        tiefe=(int)umriss.getHeight();
    }
    
    /**
     * fuegt ein Element hinzu.
     * Dabei ist zu beachten, dass nicht nur das Element hinzugefuegt wird, 
     * sondern auch seine Darstellung.
     */
    public void fuegeHinzu(Moebel moebel)
    {
        moebelListe.add(moebel);
        if (istSichtbar) moebel.zeige();
        aktualisiereUmriss();
        setzeGruppenTransformation();
    }
    
    /**
     * entfernt ein Element.
     * Wenn es dies Objekt nicht gibt, passiert nichts.
     * Dabei ist zu beachten, dass nicht nur das Element entfernt wird, 
     * sondern auch seine Darstellung.
     */
    public void entferne(Moebel moebel)
    {
        if (moebelListe.contains(moebel)){
            moebel.verberge();
            moebelListe.remove(moebel);
            aktualisiereUmriss();
            setzeGruppenTransformation();
        }
    }
    /**
     * gibt das Kindobjekt an der angegebenen Position zurueck.
     * Wenn es diese Position nicht gibt, wird null zurueckgegeben.
     */
    public Moebel gibKindObjekt(int anPosition){
        if (anPosition>=0)
        if (anPosition<moebelListe.size())
        return moebelListe.get(anPosition);
        return null;
    }    
    
    /**
    * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
    */
    protected Shape gibAktuelleFigur()
    {
        GeneralPath gruppe = new GeneralPath();
        for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
        gruppe.append(it.next().gibAktuelleFigur(), false);
        return  gruppe;
    }
    
    public void zeichne()
    {
        setzeGruppenTransformation();
        for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
        it.next().zeige();
    }

    public void loesche()
    {
        for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
        it.next().verberge();
    }

    /**
    * Aendere die Farbe dieses Objektes in 'neueFarbe'.
    * Gueltige Angaben sind "rot", "gelb", "blau", "gruen",
    * "lila" und "schwarz".
    */
    public void aendereFarbe(String neueFarbe)
    {
        loesche();
        farbe = neueFarbe;
        for (Iterator<Moebel> it=moebelListe.iterator(); it.hasNext();)
        it.next().aendereFarbe(neueFarbe);
        zeichne();
    }
    
}
