import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

/**
 * Abstrakte Klasse Moebel
 * @author Claus Albowski
 * @version  juni '09
 */

public abstract class Moebel
{
  protected int xPosition;
  protected int yPosition;
  protected int orientierung;
  protected String farbe;
  protected boolean istSichtbar;
  protected int breite;
  protected int tiefe;
  protected AffineTransform gruppenTransformation=null;

  /**
   * Hier wird mit dem Attribut istAusgewaehlt gearbeitet.
   */
  protected boolean istAusgewaehlt=false;
  
  // muss bei Gruppe ueberschrieben werden:
  protected void setzeAuswahl(boolean wert){
    istAusgewaehlt=wert;
  }
  protected String farbe(){
    if (istAusgewaehlt) return "grau";
    else return farbe;
  }

  /**
   * Berechnet das zu zeichnende Shape anhand der gegebenen Daten
   */
  protected abstract Shape gibAktuelleFigur();
  
  /**
   * Transformiert das Shape, so dass es die richtige Position
   * und Orientierung einnimmt.
   */
  protected Shape transformiere(Shape shape)
  {
      AffineTransform t = new AffineTransform();
      if (gruppenTransformation!=null) t.concatenate(gruppenTransformation);
      t.translate(xPosition, yPosition);
      Rectangle2D umriss = shape.getBounds2D();
      // relative Orientierung nach Definition:
      t.rotate(Math.toRadians(orientierung),
               umriss.getX()+umriss.getWidth()/2,
               umriss.getY()+umriss.getHeight()/2);
      // vorgegebene Rotation durch eine Gruppe:
      return  t.createTransformedShape(shape);
  }
  
    /**
     * setzt eine von der Gruppe vorgegebene Drehung.
     */
    protected void setzeGruppenTransformation(AffineTransform gruppenTransformation){
        this.gruppenTransformation=gruppenTransformation;
    }

  /**
   * Mache dieses Objekt sichtbar. Wenn es bereits sichtbar ist, tue
   * nichts.
   */
  public void zeige()
  {
    istSichtbar = true;
    zeichne();
  }

  /**
   * Mache dieses Objekt unsichtbar. Wenn es bereits unsichtbar ist, tue
   * nichts.
   */
  public void verberge()
  {
    loesche();
    istSichtbar = false;
  }

  /**
   * Drehe auf den angegebenen Winkel
   */
  public void dreheAuf(int neuerWinkel)
  {
     loesche();
     orientierung = neuerWinkel;
     zeichne();
  }

  /**
   * Bewege dieses Objekt horizontal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeHorizontal(int entfernung)
  {
    loesche();
    xPosition += entfernung;
    zeichne();
  }

  /**
   * Bewege dieses objekt vertikal um 'entfernung' Bildschirmpunkte.
   */
  public void bewegeVertikal(int entfernung)
  {
    loesche();
    yPosition += entfernung;
    zeichne();
  }
 
  /**
   * Aendere die Farbe dieses Objektes in 'neueFarbe'.
   * Gueltige Angaben sind "rot", "gelb", "blau", "gruen",
   * "lila" und "schwarz".
   */
  public void aendereFarbe(String neueFarbe)
  {
    farbe = neueFarbe;
    zeichne();
  }

  /**
   * Zeichne dieses Objekt mit seinen aktuellen Werten auf den Bildschirm.
   */
  protected void zeichne()
  {
    if (istSichtbar)
    {
      Shape figur = transformiere(gibAktuelleFigur());
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.zeichne (
        this,           // leinwand kennt das Objekt
        farbe(),          // definiert seine Zeichenfarbe  �NDERUNG F�R AUSWAHL
        figur);         // definiert seinen grafischen Aspekt
      leinwand.warte(10);
    }
  }

  /**
   * Loesche dieses Objekt vom Bildschirm.
   */
  protected void loesche()
  {
    if (istSichtbar)
    {
      Leinwand leinwand = Leinwand.gibLeinwand();
      leinwand.entferne(this);
    }
  }
}
